/*
 * glFeedback, an OpenGL Feedback processing Library
 * Copyright (C) 2003 Francesco Bradascio
 *
 * E-mail: fbradasc@katamail.com
 * URL: http://digilander.libero.it/fbradasc/GL_FEEDBACK.html
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * This work is mostly based on the GL2PS Library which is:
 *
 * Copyright (C) 1999-2003 Christophe Geuzaine
 * E-mail: geuz@geuz.org
 * URL: http://www.geuz.org/gl2ps/
 *
 * GL2PS Contributor(s):
 *   Michael Sweet <mike@easysw.com>
 *   Marc Ume <marc.ume@digitalgraphics.be>
 *   Jean-Francois Remacle <remacle@scorec.rpi.edu>
 *   Bart Kaptein <B.L.Kaptein@lumc.nl>
 *   Quy Nguyen-Dai<quy@vnilux.com>
 *   Sam Buss <sbuss@ucsd.edu>
 *   Shane Hill <Shane.Hill@dsto.defence.gov.au>
 *   Romain Boman <r_boman@yahoo.fr>
 *   Rouben Rostamian <rostamian@umbc.edu>
 *   Diego Santa Cruz <Diego.SantaCruz@epfl.ch>
 *   Shahzad Muzaffar <Shahzad.Muzaffar@cern.ch>
 *   Lassi Tuura <lassi.tuura@cern.ch>
 *   Guy Barrand <barrand@lal.in2p3.fr>
 */

#ifndef __GLFContext_H
#define __GLFContext_H

#include <GLF/glFeedback.h>
#include <GLF/GLFrgba.h>
#include <GLF/GLFVertex.h>
#include <GLF/GLFGrAttributes.h>
#include <GLF/GLFPrimitive.h>
#include <GLF/GLFPrimitives.h>
#include <GLF/GLFBuffer.h>
#include <GLF/GLFViewport.h>

// class GLFViewport;

class GLF_API GLFContext
{
friend class GLFViewport;

protected:
    GLFViewport *first;
    GLFViewport *last;

    GLFViewport *viewport;
    GLenum       parseError;

    GLint        getVertex(GLFVertex &v, GLfloat *p);

    static void  printPrimitive(GLFPrimitive& prim, void *ctx);

    virtual void writeFileHeader()=0;
    virtual void writeFrameHeader(GLint viewport[4],GLFrgba& background)=0;
    virtual void writeViewportHeader(GLint viewport[4],GLFrgba& background)=0;
    virtual void printPrimitive(GLFPrimitive& prim)=0;
    virtual void writeViewportFooter()=0;
    virtual void writeFrameFooter()=0;
    virtual void writeFileFooter()=0;
    virtual void flush()=0;

public:

    GLFBuffer       *feedback;
    GLint            buffsize;

    GLenum           colormode;
    GLenum           colorsize;
                  
    GLFrgba         *colormap;

    GLenum           options;

    char            *title;
    char            *creator;
    char            *producer;
    char            *copyright;

    GLFrgba          threshold;

    bool             boundary;

    GLint            maxbestroot;

    GLFGrAttributes  origAttributes;
    GLFGrAttributes  attributes;

    GLFContext (
        const char *title     = 0,
        const char *creator   = 0,
        const char *producer  = 0,
        const char *copyright = 0,
        GLint colormode       = GL_RGBA,
        GLint colorsize       = 0,
        GLfloat *colormap     = 0,
        GLint nr              = 0,
        GLint ng              = 0,
        GLint nb              = 0
    );
    virtual ~GLFContext();

    void         setOptions(GLint options) { this->options = options; }
    bool         setFeedbackBuffer(GLsizei buffsize=2048*2048);
    bool         active(bool skipModeCheck=false);
    GLint        fetchPrimitives();
    GLint        printPrimitives();

    void enable (                                       // as glEnable
        GLenum cap
    );
    void disable (                                      // as glDisable
        GLenum cap
    );
    void beginViewport (                                // as glViewport
        GLint viewport[4]
    );
    void endViewport (
        bool closeAll = false
    );
    void pointSize (                                    // as glPointSize
        GLfloat size
    );
    void lineWidth (                                    // as glLineWidth
        GLfloat width
    );
    void lineStipple (                                  // as glLineStipple
        GLint factor,
        GLushort pattern
    );
    void polygonOffset (                                // as glPolygonOffset
        GLfloat factor,
        GLfloat units
    );
    void polygonStipple (                               // as glPolygonStipple
        const GLubyte *mask
    );
    void drawPixels (                                   // as glDrawPixels
        GLsizei     width,
        GLsizei     height,
        GLenum      format,
        GLenum      type,
        const void *pixels
    );
};

#endif // __GLFContext_H
