/*
 * glFeedback, an OpenGL Feedback processing Library
 * Copyright (C) 2003 Francesco Bradascio
 *
 * E-mail: fbradasc@katamail.com
 * URL: http://digilander.libero.it/fbradasc/GL_FEEDBACK.html
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * This work is mostly based on the GL2PS Library which is:
 *
 * Copyright (C) 1999-2003 Christophe Geuzaine
 * E-mail: geuz@geuz.org
 * URL: http://www.geuz.org/gl2ps/
 *
 * GL2PS Contributor(s):
 *   Michael Sweet <mike@easysw.com>
 *   Marc Ume <marc.ume@digitalgraphics.be>
 *   Jean-Francois Remacle <remacle@scorec.rpi.edu>
 *   Bart Kaptein <B.L.Kaptein@lumc.nl>
 *   Quy Nguyen-Dai<quy@vnilux.com>
 *   Sam Buss <sbuss@ucsd.edu>
 *   Shane Hill <Shane.Hill@dsto.defence.gov.au>
 *   Romain Boman <r_boman@yahoo.fr>
 *   Rouben Rostamian <rostamian@umbc.edu>
 *   Diego Santa Cruz <Diego.SantaCruz@epfl.ch>
 *   Shahzad Muzaffar <Shahzad.Muzaffar@cern.ch>
 *   Lassi Tuura <lassi.tuura@cern.ch>
 *   Guy Barrand <barrand@lal.in2p3.fr>
 */

#ifndef __GLFPostscript_H
#define __GLFPostscript_H

#include <GLF/glFeedback.h>
#include <GLF/GLFrgba.h>
#include <GLF/GLFPrimitive.h>
#include <GLF/GLFContext.h>

class GLF_API GLFPostscript: public GLFContext
{
private:
    char    *filename_;
    FILE    *stream_;

    bool     eps_;
    bool     enablePs3Shading_;

    GLint    screenSize[4];

    GLFrgba  lastrgba;
    GLfloat  lastlinewidth;

    GLfloat  pageWidth_;
    GLfloat  pageHeight_;
    GLfloat  borderSize_;
    GLint    orientation_;
    GLfloat  umFactor_;

    int      pageLevel;
    int      viewportLevel;
    int      pages;

    void resetColor();

    int getRgb (
        GLfloat *pixels, GLsizei w, GLsizei h, GLuint x, GLuint y,
        GLfloat &r, GLfloat &g, GLfloat &b
    );

    int  psPrint(const char *fmt, ...);
    void writeByte(unsigned char byte);
    void writeColor(GLFrgba rgba);
    void writePixmap(GLFPrimitive& pixmap);

    // defaults uses object's umFactor_
    GLfloat toPsPoint(GLfloat v, GLfloat umFactor=0.0);

protected:
    void writeFileHeader();
    void writeFrameHeader(GLint viewport[4],GLFrgba& background);
    void writeViewportHeader(GLint viewport[4],GLFrgba& background);
    void printPrimitive(GLFPrimitive& prim);
    void writeViewportFooter();
    void writeFrameFooter();
    void writeFileFooter();
    void flush();

public:
    enum {
        PORTRAIT  = 0,
        LANDSCAPE    ,
        AS_IS
    } PageOrientations;

    GLFPostscript (
        const char *filename            ,
        GLint       screenSize[4]       ,
        const char *title            = 0,
        const char *creator          = 0,
        const char *producer         = 0,
        const char *copyright        = 0,
        bool        enablePs3Shading = true,
        GLint       colormode        = GL_RGBA,
        GLint       colorsize        = 0,
        GLfloat    *colormap         = 0,
        GLint       nr               = 0,
        GLint       ng               = 0,
        GLint       nb               = 0
    );
    ~GLFPostscript();

    void pageSetup (
        GLint     orientation,
        GLfloat   pageWidth,
        GLfloat   pageHeight,
        GLfloat   borderSize,
        GLfloat   umFactor
    );
};

#endif // __GLFPostscript_H
