/*
 * glFeedback, an OpenGL Feedback processing Library
 * Copyright (C) 2003 Francesco Bradascio
 *
 * E-mail: fbradasc@katamail.com
 * URL: http://digilander.libero.it/fbradasc/GL_FEEDBACK.html
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * This work is mostly based on the GL2PS Library which is:
 *
 * Copyright (C) 1999-2003 Christophe Geuzaine
 * E-mail: geuz@geuz.org
 * URL: http://www.geuz.org/gl2ps/
 *
 * GL2PS Contributor(s):
 *   Michael Sweet <mike@easysw.com>
 *   Marc Ume <marc.ume@digitalgraphics.be>
 *   Jean-Francois Remacle <remacle@scorec.rpi.edu>
 *   Bart Kaptein <B.L.Kaptein@lumc.nl>
 *   Quy Nguyen-Dai<quy@vnilux.com>
 *   Sam Buss <sbuss@ucsd.edu>
 *   Shane Hill <Shane.Hill@dsto.defence.gov.au>
 *   Romain Boman <r_boman@yahoo.fr>
 *   Rouben Rostamian <rostamian@umbc.edu>
 *   Diego Santa Cruz <Diego.SantaCruz@epfl.ch>
 *   Shahzad Muzaffar <Shahzad.Muzaffar@cern.ch>
 *   Lassi Tuura <lassi.tuura@cern.ch>
 *   Guy Barrand <barrand@lal.in2p3.fr>
 */

#ifndef __GLFPrimitive_H
#define __GLFPrimitive_H

#include <GLF/glFeedback.h>
#include <GLF/GLFGrAttributes.h>
#include <GLF/GLFplane.h>
#include <GLF/GLFVertex.h>
#include <GLF/GLFVertices.h>

class GLF_API GLFPrimitive
{
private:
    GLFPrimitive *t1_;
    GLFPrimitive *t2_;

protected:
    GLFPrimitive *piece(const GLFplane& plane, bool isFrontPiece=true);
    GLFPrimitive *piece2D(const GLFplane& plane, bool isFrontPiece=true);

public:
    static unsigned long id;

    unsigned long    count;

    GLFGrAttributes  attributes;
    GLint            type;
    GLFVertices      verts;
    GLubyte          culled;

    GLsizei          width;
    GLsizei          height;
    GLenum           format;
    GLenum           imageType;
    GLfloat         *pixels;

    GLFPrimitive (
        GLint type,
        GLFVertex *verts,
        GLFGrAttributes &attributes,
        GLenum format=0, GLenum imageType=0,
        GLsizei width=0, GLsizei height=0,
        GLfloat *pixels=0
    );
    GLFPrimitive(const GLFPrimitive &p);
    GLFPrimitive(GLint type, const GLFVertices &verts);
    GLFPrimitive(GLint type, GLFVertex *verts, const GLFPrimitive &p);
    GLFPrimitive(GLint type, const GLFVertices &verts, const GLFPrimitive &p);

    ~GLFPrimitive();

    GLint vertices();

    GLFPrimitive& operator=(const GLFPrimitive& b);
    const GLFplane &plane();

    GLFPrimitive *t1();
    GLFPrimitive *t2();

    GLint classify(const GLFplane &plane);
    GLFPrimitive *frontPiece(const GLFplane &plane);
    GLFPrimitive *backPiece(const GLFplane &plane);
    GLFPrimitive *frontPiece2D(const GLFplane &plane);
    GLFPrimitive *backPiece2D(const GLFplane &plane);

    static int gtcType  (const GLFPrimitive &pa, const GLFPrimitive &pb);
    static int ltcType  (const GLFPrimitive &pa, const GLFPrimitive &pb);
    GLint check(const GLFplane &plane);
    GLboolean sameVertexColor();
};

#endif // __GLFPrimitive_H
