/*
 * glFeedback, an OpenGL Feedback processing Library
 * Copyright (C) 2003 Francesco Bradascio
 *
 * E-mail: fbradasc@katamail.com
 * URL: http://digilander.libero.it/fbradasc/GL_FEEDBACK.html
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * This work is mostly based on the GL2PS Library which is:
 *
 * Copyright (C) 1999-2003 Christophe Geuzaine
 * E-mail: geuz@geuz.org
 * URL: http://www.geuz.org/gl2ps/
 *
 * GL2PS Contributor(s):
 *   Michael Sweet <mike@easysw.com>
 *   Marc Ume <marc.ume@digitalgraphics.be>
 *   Jean-Francois Remacle <remacle@scorec.rpi.edu>
 *   Bart Kaptein <B.L.Kaptein@lumc.nl>
 *   Quy Nguyen-Dai<quy@vnilux.com>
 *   Sam Buss <sbuss@ucsd.edu>
 *   Shane Hill <Shane.Hill@dsto.defence.gov.au>
 *   Romain Boman <r_boman@yahoo.fr>
 *   Rouben Rostamian <rostamian@umbc.edu>
 *   Diego Santa Cruz <Diego.SantaCruz@epfl.ch>
 *   Shahzad Muzaffar <Shahzad.Muzaffar@cern.ch>
 *   Lassi Tuura <lassi.tuura@cern.ch>
 *   Guy Barrand <barrand@lal.in2p3.fr>
 */

#ifndef GL_FEEDBACK_H
#define GL_FEEDBACK_H

#include <stdio.h>
#include <math.h>
#include <assert.h>

#ifdef WIN32
# include <windows.h>
#endif
#ifndef GLF_API
# if !defined(WIN32) || !defined(GLF_SHARED)
#   define GLF_API
# else
#   ifdef GLF_EXPORTS
#     define GLF_API __declspec(dllexport)
#   else
#     define GLF_API __declspec(dllimport)
#   endif
# endif
#endif

#ifdef __APPLE__
#  include <OpenGL/gl.h>
#else
#  include <GL/gl.h>
#endif

#ifndef __cplusplus

extern void glfBeginFrame(GLint x, GLint y, GLsizei width, GLsizei height);

extern void glfViewport(GLint x, GLint y, GLsizei width, GLsizei height);
extern GLint glfDrawPixels (
    GLsizei width , GLsizei height,
    GLenum  format, GLenum  type  ,
    const void *pixels
);
extern void glfPointSize(GLfloat size);
extern void glfLineWidth(GLfloat width);
extern void glfLineStipple(GLint factor, GLushort pattern);
extern void glfPolygonStipple(const GLubyte *mask);
extern void glfEnable(GLenum cap);
extern void glfDisable(GLenum cap);

extern void glfEndFrame();

#else // __cplusplus

extern "C" {
    void glfBeginFrame(GLint x, GLint y, GLsizei width, GLsizei height);

    void glfViewport(GLint x, GLint y, GLsizei width, GLsizei height);
    void glfDrawPixels (
        GLsizei width , GLsizei height,
        GLenum  format, GLenum  type  ,
        const void *pixels
    );
    void glfPointSize(GLfloat size);
    void glfLineWidth(GLfloat width);
    void glfLineStipple(GLint factor, GLushort pattern);
    void glfPolygonStipple(const GLubyte *mask);
    void glfEnable(GLenum cap);
    void glfDisable(GLenum cap);

    void glfEndFrame();
}

/* Sorting algorithms */

enum GLF_Options {                  // Options for glfBeginPage
    GLF_DRAW_BACKGROUND    = 0,
    GLF_SIMPLE_LINE_OFFSET    ,
    GLF_BEST_ROOT             ,
    GLF_OCCLUSION_CULL        ,
    GLF_NO_PS3_SHADING
};

#define GLF_OPTION_FLAG(a)     (1<<(a))

enum GLF_Enable {                   // Arguments for glfEnable/glfDisable
    GLF_POLYGON_OFFSET_FILL = 1,
    GLF_POLYGON_BOUNDARY       ,
    GLF_LINE_STIPPLE
};

/* Magic numbers */

#define GLF_EPSILON             5.e-3
#define GLF_DEPTH_FACT          1000.0
#define GLF_SIMPLE_OFFSET       0.05
#define GLF_ZERO(arg)           (fabs(arg)<1.e-20)

enum GLF_ErrorCodes {
    GLF_SUCCESS       = 0,
    GLF_NO_FEEDBACK      ,
    GLF_OVERFLOW
};

enum GLF_PrimitiveTypes {           // Primitive types
    GLF_PIXMAP     = 0,
    GLF_POINT         ,
    GLF_LINE          ,
    GLF_TRIANGLE      ,
    GLF_QUADRANGLE    ,
    GLF_POLYGON
};

enum GLF_BspTree3D {                // BSP tree primitive comparison
    GLF_COINCIDENT  = 1,
    GLF_IN_FRONT_OF    ,
    GLF_IN_BACK_OF     ,
    GLF_SPANNING
};

enum GLF_bspTree2D {                // 2D BSP tree primitive comparison
    GLF_POINT_COINCIDENT = 0,
    GLF_POINT_INFRONT       ,
    GLF_POINT_BACK
};

enum GLF_PassThroughOptions {       // Pass through options
    GLF_BEGIN_POLYGON_OFFSET_FILL = 1,
    GLF_END_POLYGON_OFFSET_FILL      ,
    GLF_BEGIN_POLYGON_BOUNDARY       ,
    GLF_END_POLYGON_BOUNDARY         ,
    GLF_SET_POLYGON_STIPPLE          ,
    GLF_CLEAR_POLYGON_STIPPLE        ,
    GLF_SET_LINE_STIPPLE             ,
    GLF_CLEAR_LINE_STIPPLE           ,
    GLF_SET_POINT_SIZE               ,
    GLF_SET_LINE_WIDTH
};

class GLFContext;

extern void glfSetContext (GLFContext *ctx, GLsizei buffsize);

#endif // __cplusplus
#endif // GL_FEEDBACK_H
