/*
 * glFeedback, an OpenGL Feedback processing Library
 * Copyright (C) 2003 Francesco Bradascio
 *
 * E-mail: fbradasc@katamail.com
 * URL: http://digilander.libero.it/fbradasc/GL_FEEDBACK.html
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * This work is mostly based on the GL2PS Library which is:
 *
 * Copyright (C) 1999-2003 Christophe Geuzaine
 * E-mail: geuz@geuz.org
 * URL: http://www.geuz.org/gl2ps/
 *
 * GL2PS Contributor(s):
 *   Michael Sweet <mike@easysw.com>
 *   Marc Ume <marc.ume@digitalgraphics.be>
 *   Jean-Francois Remacle <remacle@scorec.rpi.edu>
 *   Bart Kaptein <B.L.Kaptein@lumc.nl>
 *   Quy Nguyen-Dai<quy@vnilux.com>
 *   Sam Buss <sbuss@ucsd.edu>
 *   Shane Hill <Shane.Hill@dsto.defence.gov.au>
 *   Romain Boman <r_boman@yahoo.fr>
 *   Rouben Rostamian <rostamian@umbc.edu>
 *   Diego Santa Cruz <Diego.SantaCruz@epfl.ch>
 *   Shahzad Muzaffar <Shahzad.Muzaffar@cern.ch>
 *   Lassi Tuura <lassi.tuura@cern.ch>
 *   Guy Barrand <barrand@lal.in2p3.fr>
 */

#include <GLF/GLFMath.h>

void GLFMath::normal(GLFxyz &va, GLFxyz &vb, GLFxyz &vr)
{
    GLFMath::pvect(va,vb,vr);

    if (GLF_ZERO(GLFMath::normalize(vr))) {
        // FIXME: the plane is still wrong... Let's return a dummy value
        // for now (this is a hack: we should do more tests in GetPlane)
        vr[0] = vr[1] = 0.0;
        vr[2] = 1.0;
    }
}

GLfloat GLFMath::normalize(GLFxyz& v)
{
GLfloat norm;

    norm = sqrt( v[0]*v[0] + v[1]*v[1] + v[2]*v[2] );
    if (!GLF_ZERO(norm)) {
        v[0] /= norm;
        v[1] /= norm;
        v[2] /= norm;
    }
    return norm;
}

GLfloat GLFMath::pscal(GLFxyz &va, GLFxyz &vb)
{
    return ( va[0]*vb[0] + va[1]*vb[1] + va[2]*vb[2] );
}

void GLFMath::pvect(GLFxyz &va, GLFxyz &vb, GLFxyz &vr)
{
    vr[0] = va[1]*vb[2] - va[2]*vb[1];
    vr[1] = va[2]*vb[0] - va[0]*vb[2];
    vr[2] = va[0]*vb[1] - va[1]*vb[0];
}

GLboolean GLFMath::greater(GLfloat f1, GLfloat f2)
{
  return (f1 > f2) ? 1 : 0;
}

GLboolean GLFMath::less(GLfloat f1, GLfloat f2)
{
  return (f1 < f2) ? 1 : 0;
}
