/*
 * glFeedback, an OpenGL Feedback processing Library
 * Copyright (C) 2003 Francesco Bradascio
 *
 * E-mail: fbradasc@katamail.com
 * URL: http://digilander.libero.it/fbradasc/GL_FEEDBACK.html
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * This work is mostly based on the GL2PS Library which is:
 *
 * Copyright (C) 1999-2003 Christophe Geuzaine
 * E-mail: geuz@geuz.org
 * URL: http://www.geuz.org/gl2ps/
 *
 * GL2PS Contributor(s):
 *   Michael Sweet <mike@easysw.com>
 *   Marc Ume <marc.ume@digitalgraphics.be>
 *   Jean-Francois Remacle <remacle@scorec.rpi.edu>
 *   Bart Kaptein <B.L.Kaptein@lumc.nl>
 *   Quy Nguyen-Dai<quy@vnilux.com>
 *   Sam Buss <sbuss@ucsd.edu>
 *   Shane Hill <Shane.Hill@dsto.defence.gov.au>
 *   Romain Boman <r_boman@yahoo.fr>
 *   Rouben Rostamian <rostamian@umbc.edu>
 *   Diego Santa Cruz <Diego.SantaCruz@epfl.ch>
 *   Shahzad Muzaffar <Shahzad.Muzaffar@cern.ch>
 *   Lassi Tuura <lassi.tuura@cern.ch>
 *   Guy Barrand <barrand@lal.in2p3.fr>
 */

#include <GLF/GLFPolygonAttributes.h>

GLFPolygonAttributes::GLFPolygonAttributes()
{
    offset      = 0;
    boundary    = 0;
    stippleMask = 0;
}

GLFPolygonAttributes::GLFPolygonAttributes(const GLFPolygonAttributes& p)
{
GLFPolygonAttributes& me = *this;

    offset      = 0;
    stippleMask = 0;
    me          = p;
}

GLFPolygonAttributes::~GLFPolygonAttributes()
{
    if (stippleMask) {
        delete stippleMask;
    }
    if (offset) {
        delete offset;
    }
}

GLFPolygonAttributes& GLFPolygonAttributes::operator= (
    const GLFPolygonAttributes& p
) {
int i;

    if (&p != this) {
        if (offset) {
            delete offset;
        }
        if (stippleMask) {
            delete stippleMask;
        }
        offset      = 0;
        stippleMask = 0;
        boundary    = p.boundary;
        if (p.offset) {
            offset = new GLfloat[2];
            for (i=0; i<2; i++) {
                offset[i] = p.offset[i];
            }
        }
        if (p.stippleMask) {
            stippleMask = new GLfloat[32];
            for (i=0; i<32; i++) {
                stippleMask[i] = p.stippleMask[i];
            }
        }
    }
    return *this;
}
