/*
 * glFeedback, an OpenGL Feedback processing Library
 * Copyright (C) 2003 Francesco Bradascio
 *
 * E-mail: fbradasc@katamail.com
 * URL: http://digilander.libero.it/fbradasc/GL_FEEDBACK.html
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * This work is mostly based on the GL2PS Library which is:
 *
 * Copyright (C) 1999-2003 Christophe Geuzaine
 * E-mail: geuz@geuz.org
 * URL: http://www.geuz.org/gl2ps/
 *
 * GL2PS Contributor(s):
 *   Michael Sweet <mike@easysw.com>
 *   Marc Ume <marc.ume@digitalgraphics.be>
 *   Jean-Francois Remacle <remacle@scorec.rpi.edu>
 *   Bart Kaptein <B.L.Kaptein@lumc.nl>
 *   Quy Nguyen-Dai<quy@vnilux.com>
 *   Sam Buss <sbuss@ucsd.edu>
 *   Shane Hill <Shane.Hill@dsto.defence.gov.au>
 *   Romain Boman <r_boman@yahoo.fr>
 *   Rouben Rostamian <rostamian@umbc.edu>
 *   Diego Santa Cruz <Diego.SantaCruz@epfl.ch>
 *   Shahzad Muzaffar <Shahzad.Muzaffar@cern.ch>
 *   Lassi Tuura <lassi.tuura@cern.ch>
 *   Guy Barrand <barrand@lal.in2p3.fr>
 */

#include <string.h>

#include <GLF/GLFPrimitive.h>
#include <GLF/GLFMath.h>

unsigned long GLFPrimitive::id = 0;

GLFPrimitive *GLFPrimitive::piece(const GLFplane& pl, bool isFrontPiece)
{
GLFPrimitive *p = new GLFPrimitive(-1,0,*this);

#if 0
GLsizei nverts;
GLint ptA;
GLfloat sideA, sideB;

    nverts   = verts.size();
    ptA      = nverts - 1;
    sideA    = pl.compare(verts[ptA].xyz);

    p->attributes.polygon.boundary = 0; /* not done! */
    p->verts.clear();

    for (int ptB=0; (ptB<nverts) && (p->verts.size()<4); ptB++) {
        sideB = pl.compare(verts[ptB].xyz);
        if (sideB > +GLF_EPSILON) {
            if (sideA < -GLF_EPSILON) {
                // compute the intersection point of the line
                // from point A to point B with the partition
                // plane. This is a simple ray-plane intersection.
                p->verts.push_back(verts[ptA].cutEdge(verts[ptB],pl));
            }
            if (!isFrontPiece && (p->verts.size()<4)) {
                p->verts.push_back(verts[ptB]);
            }
        } else if (sideB < -GLF_EPSILON) {
            if (sideA > +GLF_EPSILON) {
                // compute the intersection point of the line
                // from point A to point B with the partition
                // plane. This is a simple ray-plane intersection.
                p->verts.push_back(verts[ptA].cutEdge(verts[ptB],pl));
            }
            if (isFrontPiece && (p->verts.size()<4)) {
                p->verts.push_back(verts[ptB]);
            }
        } else {
            p->verts.push_back(verts[ptB]);
        }
        ptA   = ptB;
        sideA = sideB;
    }
#else
GLshort i, j;
GLint nverts = verts.size();
GLfloat d[5];

    d[0] = pl.compare(verts[0].xyz);
    for (i = 0; i < nverts && (p->verts.size()<4); i++) {
        if (i < nverts-1) {
            j = i + 1;
            d[j] = pl.compare(verts[j].xyz);
        } else {
            j = 0;
        }
        if (d[j] > GLF_EPSILON) {
            if (d[i] < -GLF_EPSILON) {
                // compute the intersection point of the line
                // from point A to point B with the partition
                // plane. This is a simple ray-plane intersection.
                p->verts.push_back(verts[i].cutEdge(verts[j],pl));
            }
            if (!isFrontPiece && (p->verts.size()<4)) {
                p->verts.push_back(verts[j]);
            }
        } else if (d[j] < -GLF_EPSILON) {
            if (d[i] > +GLF_EPSILON) {
                // compute the intersection point of the line
                // from point A to point B with the partition
                // plane. This is a simple ray-plane intersection.
                p->verts.push_back(verts[i].cutEdge(verts[j],pl));
            }
            if (isFrontPiece && (p->verts.size()<4)) {
                p->verts.push_back(verts[j]);
            }
        } else {
            p->verts.push_back(verts[j]);
        }
    }
#endif
    p->type = p->verts.size();

    return p;
}

GLFPrimitive::GLFPrimitive (
    GLint type,
    GLFVertex *verts,
    GLFGrAttributes &attributes,
    GLenum format, GLenum imageType,
    GLsizei width, GLsizei height,
    GLfloat *pixels
) {
GLint i, vnum; 

    this->count = GLFPrimitive::id++;

    this->type       = type;
    this->attributes = attributes;
    this->format     = format;
    this->imageType  = imageType;
    this->width      = width;
    this->height     = height;
    this->pixels     = 0;
    if (pixels && width*height>0) {
        int size      = width*height*3;
        this->pixels  = new GLfloat[size];
        size         *= sizeof(GLfloat);
        memcpy(this->pixels,pixels,size);
    }
    if (verts) {
        for (i=0, vnum=vertices(); i<vnum; i++) {
            this->verts.push_back(verts[i]);
        }
    }
    this->t1_        = 0;
    this->t2_        = 0;
    this->culled     = 0;
}

GLFPrimitive::GLFPrimitive(const GLFPrimitive &p)
{
    this->count = GLFPrimitive::id++;

    t1_                  = 0;
    t2_                  = 0;
    type                 = -1;
    culled               = 0;
    format               = 0;
    imageType            = 0;
    width                = 0;
    height               = 0;
    pixels               = 0;
    (GLFPrimitive&)*this = p;
}

GLFPrimitive::GLFPrimitive(GLint type, GLFVertex *verts, const GLFPrimitive &p)
{
GLint i, vnum;

    this->count = GLFPrimitive::id++;

    this->t1_            = 0;
    this->t2_            = 0;
    this->culled         = 0;
    this->format         = 0;
    this->imageType      = 0;
    this->width          = 0;
    this->height         = 0;
    this->pixels         = 0;
    this->type           = (verts) ? -2 : type; // avoid copy of p's vertices
    (GLFPrimitive&)*this = p;
    this->type           = type;
    this->verts.clear();
    if (verts) {
        for (i=0, vnum=vertices(); i<vnum; i++) {
            this->verts.push_back(verts[i]);
        }
    }
}

GLFPrimitive::GLFPrimitive(GLint type,const GLFVertices& verts)
{
    this->count = GLFPrimitive::id++;

    this->t1_       = 0;
    this->t2_       = 0;
    this->type      = type;
    this->culled    = 0;
    this->format    = 0;
    this->imageType = 0;
    this->width     = 0;
    this->height    = 0;
    this->pixels    = 0;
    this->verts     = verts;
}

GLFPrimitive::GLFPrimitive (
    GLint type,
    const GLFVertices& verts,
    const GLFPrimitive &p
) {
    this->count = GLFPrimitive::id++;

    this->t1_            = 0;
    this->t2_            = 0;
    this->type           = type;
    this->culled         = 0;
    this->format         = 0;
    this->imageType      = 0;
    this->width          = 0;
    this->height         = 0;
    this->pixels         = 0;
    (GLFPrimitive&)*this = p;
    this->verts          = verts;
}

GLFPrimitive::~GLFPrimitive()
{
    if (pixels) {
        delete pixels;
    }
    if (t1_) {
        delete t1_;
    }
    if (t2_) {
        delete t2_;
    }
}

GLint GLFPrimitive::vertices()
{
    return (type == GLF_POINT     ) ? 1 :
           (type == GLF_PIXMAP    ) ? 1 :
           (type == GLF_LINE      ) ? 2 :
           (type == GLF_TRIANGLE  ) ? 3 :
           (type == GLF_QUADRANGLE) ? 4 : 0;
}

GLFPrimitive& GLFPrimitive::operator=(const GLFPrimitive& p)
{
    if (&p != this) {
        attributes = p.attributes;
        type = (type==-1) ? p.type : type;
        if (type>=0 && type == p.type) {
            verts  = p.verts;
        }
        culled     = p.culled;

        format     = p.format;   
        imageType  = p.imageType;
        width      = p.width;
        height     = p.height;
        if (pixels) {
            delete pixels;
        }
        pixels = 0;
        if (p.pixels && p.width*p.height>0) {
            int size  = p.width*p.height*3;
            pixels    = new GLfloat[size];
            size     *= sizeof(GLfloat);
            memcpy(pixels,p.pixels,size);
        }
    }
    return *this;
}

const GLFplane& GLFPrimitive::plane()
{
static GLFplane p(0.0,0.0,1.0,0.0);

    switch (type) {
        case GLF_POINT:
        case GLF_PIXMAP:
                p.set(0.0,0.0,1.0,-verts[0].xyz[2]);
            break;
        case GLF_LINE:
            {
            GLFxyz v(0.0,0.0,0.0);
            GLFxyz w(0.0,0.0,0.0);
            GLFxyz k(0.0,0.0,0.0);
        
                v[0] = verts[1].xyz[0] - verts[0].xyz[0];
                v[1] = verts[1].xyz[1] - verts[0].xyz[1];
                v[2] = verts[1].xyz[2] - verts[0].xyz[2];
        
                if ( GLF_ZERO(v[0]) && GLF_ZERO(v[1]) && GLF_ZERO(v[2]) ) {
                    p[0] = p[1] = 0.0;
                    p[2] = 1.0;
                    p[3] = -verts[0].xyz[2];
                } else {
                    if (GLF_ZERO(v[0])) {
                        w[0] = 1.0;
                    } else if (GLF_ZERO(v[1]))  {
                        w[1] = 1.0;
                    } else {
                        w[2] = 1.0;
                    }
                    GLFMath::normal(v,w,k);
                    p[0] = k[0]; p[1] = k[1]; p[2] = k[2];
                    p[3] = - p[0]*verts[0].xyz[0]
                           - p[1]*verts[0].xyz[1]
                           - p[2]*verts[0].xyz[2];
                }
            }
            break;
        case GLF_TRIANGLE:
        case GLF_QUADRANGLE:
            {
            GLFxyz v(0.0,0.0,0.0);
            GLFxyz w(0.0,0.0,0.0);
            GLFxyz k(0.0,0.0,0.0);
        
                v[0] = verts[1].xyz[0] - verts[0].xyz[0];
                v[1] = verts[1].xyz[1] - verts[0].xyz[1];
                v[2] = verts[1].xyz[2] - verts[0].xyz[2];
                w[0] = verts[2].xyz[0] - verts[0].xyz[0];
                w[1] = verts[2].xyz[1] - verts[0].xyz[1];
                w[2] = verts[2].xyz[2] - verts[0].xyz[2];
                if (
                    (GLF_ZERO(v[0]) && GLF_ZERO(v[1]) && GLF_ZERO(v[2])) ||
                    (GLF_ZERO(w[0]) && GLF_ZERO(w[1]) && GLF_ZERO(w[2]))
                ) {
                    p[0] = p[1] = 0.0;
                    p[2] = 1.0;
                    p[3] = -verts[0].xyz[2];
                } else{
                    GLFMath::normal(v,w,k);
                    p[0] = k[0]; p[1] = k[1]; p[2] = k[2];
                    p[3] = - p[0]*verts[0].xyz[0]
                           - p[1]*verts[0].xyz[1]
                           - p[2]*verts[0].xyz[2];
                }
            }
            break;
    }
    return p;
}

GLint GLFPrimitive::classify(const GLFplane &plane)
{
GLshort i, j;
GLint type, nverts = verts.size();
GLfloat d[5];

    type = GLF_COINCIDENT;
    if (nverts<2) {
        d[0] = plane.compare(verts[0].xyz);
        type = ( d[0] >  GLF_EPSILON ) ? GLF_IN_BACK_OF  :
               ( d[0] < -GLF_EPSILON ) ? GLF_IN_FRONT_OF : type ;
    } else {
        d[0] = plane.compare(verts[0].xyz);
        for (i = 0; i < nverts; i++) {
            if (i < nverts-1) {
                j = i + 1;
                d[j] = plane.compare(verts[j].xyz);
            } else {
                j = 0;
            }
            if (d[j] > GLF_EPSILON) {
                type = ( type  == GLF_COINCIDENT  ) ? GLF_IN_BACK_OF  :
                       ( type  != GLF_IN_BACK_OF  ) ? GLF_SPANNING    : type ;
                type = ( d[i]   < -GLF_EPSILON    ) ? GLF_SPANNING    : type ;
            } else if (d[j] < -GLF_EPSILON) {
                type = ( type  == GLF_COINCIDENT  ) ? GLF_IN_FRONT_OF :
                       ( type  != GLF_IN_FRONT_OF ) ? GLF_SPANNING    : type ;
                type = ( d[i]   > GLF_EPSILON     ) ? GLF_SPANNING    : type ;
            }
        }
    }
    return type;
}

GLFPrimitive *GLFPrimitive::frontPiece(const GLFplane &plane)
{
    return piece(plane);
}

GLFPrimitive *GLFPrimitive::backPiece(const GLFplane &plane)
{
    return piece(plane,false);
}

GLFPrimitive *GLFPrimitive::piece2D (
    const GLFplane &plane,
    bool isFrontPiece
) {
GLFPrimitive *p = new GLFPrimitive(-1,0,*this);
//
// cur will hold the position of the current vertex
// prev will hold the position of the previous vertex
// prev0 will hold the position of the vertex number 0
// v1 and v2 represent the current and previous vertices, respectively
// flag is set if the current vertex should be checked against the plane
//
GLint cur       = -1;
GLint prev      = -1;
unsigned int v1 =  0;
unsigned int v2 =  0;
GLint flag      =  1;
GLint prev0     = -1;
unsigned int i;

    p->verts.clear();
    for (i=0; (i<=verts.size()) && (p->verts.size()<4); i++) {
        v1 = i;
        if (v1==verts.size()) {
            if (verts.size()<3) {
                break;
            }
            v1  = 0;
            v2  = verts.size()-1;
            cur = prev0;
        } else if (flag) {
            cur   = verts[v1].check(plane);
            prev0 = (i==0) ? cur : prev0;
        }
        if (((prev==-1) || (prev==cur) || !prev || !cur) && (i<verts.size())) {
            if (
                (cur==GLF_POINT_INFRONT && isFrontPiece       ) ||
                (cur==GLF_POINT_BACK    && !isFrontPiece      ) ||
                (cur!=GLF_POINT_INFRONT && cur!=GLF_POINT_BACK)
            ) {
                p->verts.push_back(verts[v1]);
            }
            flag = 1;
        } else if ((prev!=cur) && cur && prev) {
            if (v1) {
                v2 = v1-1;
                i--;
            }
            p->verts.push_back(verts[v2].cutEdge(verts[v1],plane));
            flag = 0;
        }
        prev = cur;
    }
    p->attributes.polygon.boundary = 0; /* not done! */
    p->type = p->verts.size();
    return p;
}

GLFPrimitive *GLFPrimitive::frontPiece2D(const GLFplane& plane)
{
    return piece2D(plane);
}

GLFPrimitive *GLFPrimitive::backPiece2D(const GLFplane& plane)
{
    return piece2D(plane,false);
}

int GLFPrimitive::gtcType(const GLFPrimitive &a, const GLFPrimitive &b)
{
    return ( a.type > b.type ) ? 1 : ( a.type == b.type ) ? 0 : -1 ;
}

int GLFPrimitive::ltcType(const GLFPrimitive &a, const GLFPrimitive &b)
{
    return ( a.type < b.type ) ? 1 : ( a.type == b.type ) ? 0 : -1 ;
}

GLint GLFPrimitive::check(const GLFplane& plane)
{
GLint pos;

    pos = verts[0].check(plane);
    for (unsigned int i = 1; i < verts.size(); i++) {
        pos |= verts[i].check(plane);
        if (pos == (GLF_POINT_INFRONT | GLF_POINT_BACK)) {
            return GLF_SPANNING;
        }
    }
    return (pos & GLF_POINT_INFRONT) ? GLF_IN_FRONT_OF :
           (pos & GLF_POINT_BACK   ) ? GLF_IN_BACK_OF  : GLF_COINCIDENT;
}

GLboolean GLFPrimitive::sameVertexColor()
{
    for (unsigned int i=1; i<verts.size(); i++) {
        if (verts[0].rgba != verts[i].rgba) {
            return 0;
        }
    }
    return 1;
}

GLFPrimitive *GLFPrimitive::t1()
{
GLFVertex v[3];
GLFGrAttributes attrib(attributes);

    if (t1_) {
        delete t1_;
    }
    v[0] = verts[0];
    v[1] = verts[1];
    v[2] = verts[2];
    attrib.polygon.boundary = (attributes.polygon.boundary & 1) ? 1 : 0 |
                              (attributes.polygon.boundary & 2) ? 2 : 0 ;

    t1_ = new GLFPrimitive(GLF_TRIANGLE,v,*this);
    t1_->attributes = attrib;
    return t1_;
}

GLFPrimitive *GLFPrimitive::t2()
{
GLFVertex v[3];
GLFGrAttributes attrib(attributes);

    if (t2_) {
        delete t1_;
    }
    v[0] = verts[0];
    v[1] = verts[2];
    v[2] = verts[3];
    attrib.polygon.boundary = (attributes.polygon.boundary & 4) ? 2 : 0 |
                              (attributes.polygon.boundary & 4) ? 2 : 0 ;

    t2_ = new GLFPrimitive(GLF_TRIANGLE,v,*this);
    t2_->attributes = attrib;
    return t2_;
}
