/*
 * glFeedback, an OpenGL Feedback processing Library
 * Copyright (C) 2003 Francesco Bradascio
 *
 * E-mail: fbradasc@katamail.com
 * URL: http://digilander.libero.it/fbradasc/GL_FEEDBACK.html
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * This work is mostly based on the GL2PS Library which is:
 *
 * Copyright (C) 1999-2003 Christophe Geuzaine
 * E-mail: geuz@geuz.org
 * URL: http://www.geuz.org/gl2ps/
 *
 * GL2PS Contributor(s):
 *   Michael Sweet <mike@easysw.com>
 *   Marc Ume <marc.ume@digitalgraphics.be>
 *   Jean-Francois Remacle <remacle@scorec.rpi.edu>
 *   Bart Kaptein <B.L.Kaptein@lumc.nl>
 *   Quy Nguyen-Dai<quy@vnilux.com>
 *   Sam Buss <sbuss@ucsd.edu>
 *   Shane Hill <Shane.Hill@dsto.defence.gov.au>
 *   Romain Boman <r_boman@yahoo.fr>
 *   Rouben Rostamian <rostamian@umbc.edu>
 *   Diego Santa Cruz <Diego.SantaCruz@epfl.ch>
 *   Shahzad Muzaffar <Shahzad.Muzaffar@cern.ch>
 *   Lassi Tuura <lassi.tuura@cern.ch>
 *   Guy Barrand <barrand@lal.in2p3.fr>
 */

#include <GLF/GLFVertex.h>
#include <GLF/GLFMath.h>

GLFVertex::GLFVertex()
{
}

GLFVertex::GLFVertex(const GLFVertex &v)
{
    xyz  = v.xyz;
    rgba = v.rgba;
}

GLFVertex& GLFVertex::operator=(const GLFVertex &b)
{
    if (&b != this) {
        xyz  = b.xyz;
        rgba = b.rgba;
    }
    return *this;
}

GLFVertex::GLFVertex (
    GLfloat x, GLfloat y, GLfloat z,
    GLfloat r, GLfloat g, GLfloat b, GLfloat a
) {
    xyz[0]  = x; xyz[1]  = y; xyz[2]  = z;
    rgba[0] = r; rgba[1] = g; rgba[2] = b; rgba[3] = a;
}

const GLFVertex& GLFVertex::cutEdge(const GLFVertex &b, const GLFplane &plane)
{
GLFxyz v;
GLfloat sect;
static GLFVertex vertex;

    v[0] = b.xyz[0] - xyz[0];
    v[1] = b.xyz[1] - xyz[1];
    v[2] = b.xyz[2] - xyz[2];

    sect = - plane.compare(xyz) / GLFMath::pscal((GLFxyz&)plane,v);

    vertex.xyz[0]  = xyz[0] + v[0] * sect;
    vertex.xyz[1]  = xyz[1] + v[1] * sect;
    vertex.xyz[2]  = xyz[2] + v[2] * sect;
    vertex.rgba[0] = (1.0-sect) * rgba[0] + sect * b.rgba[0];
    vertex.rgba[1] = (1.0-sect) * rgba[1] + sect * b.rgba[1];
    vertex.rgba[2] = (1.0-sect) * rgba[2] + sect * b.rgba[2];
    vertex.rgba[3] = (1.0-sect) * rgba[3] + sect * b.rgba[3];

    return vertex;
}

GLint GLFVertex::check(const GLFplane &plane)
{
GLfloat pt_dis = plane.compare(xyz);

    return (pt_dis > +GLF_EPSILON) ? GLF_POINT_INFRONT :
           (pt_dis < -GLF_EPSILON) ? GLF_POINT_BACK    : GLF_POINT_COINCIDENT;
}
