/*
 * glFeedback, an OpenGL Feedback processing Library
 * Copyright (C) 2003 Francesco Bradascio
 *
 * E-mail: fbradasc@katamail.com
 * URL: http://digilander.libero.it/fbradasc/GL_FEEDBACK.html
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * This work is mostly based on the GL2PS Library which is:
 *
 * Copyright (C) 1999-2003 Christophe Geuzaine
 * E-mail: geuz@geuz.org
 * URL: http://www.geuz.org/gl2ps/
 *
 * GL2PS Contributor(s):
 *   Michael Sweet <mike@easysw.com>
 *   Marc Ume <marc.ume@digitalgraphics.be>
 *   Jean-Francois Remacle <remacle@scorec.rpi.edu>
 *   Bart Kaptein <B.L.Kaptein@lumc.nl>
 *   Quy Nguyen-Dai<quy@vnilux.com>
 *   Sam Buss <sbuss@ucsd.edu>
 *   Shane Hill <Shane.Hill@dsto.defence.gov.au>
 *   Romain Boman <r_boman@yahoo.fr>
 *   Rouben Rostamian <rostamian@umbc.edu>
 *   Diego Santa Cruz <Diego.SantaCruz@epfl.ch>
 *   Shahzad Muzaffar <Shahzad.Muzaffar@cern.ch>
 *   Lassi Tuura <lassi.tuura@cern.ch>
 *   Guy Barrand <barrand@lal.in2p3.fr>
 */

#include <stdlib.h>

#include <GLF/GLFVertices.h>

GLFVertices::GLFVertices()
{
    vect_        = 0;
    size_        = 0;
    capacity_    = 0;
    resize(0);
}

GLFVertices::GLFVertices(const GLFVertices& v)
{
    vect_        = 0;
    size_        = 0;
    capacity_    = 0;
    (GLFVertices&)*this = v;
}

GLFVertices::~GLFVertices()
{
    if (vect_) {
        for (unsigned int i=0; i<size_; i++) {
            if (vect_[i]) {
                delete vect_[i];
            }
        }
        delete vect_;
    }
}

GLFVertices& GLFVertices::operator=(const GLFVertices& v)
{
unsigned int size = v.size();

    resize(size);
    for (unsigned int i=0; i<size; i++) {
        vect_[i] = new GLFVertex(v[i]);
    }
    return *this;
}

size_t GLFVertices::size() const
{
    return size_;
}

GLFVertex& GLFVertices::operator [] (size_t i)
{
    // return *(vect_[(i<0 || i>=size_) ? 0 : i]);
    return *(vect_[i]);
}

GLFVertex& GLFVertices::operator [] (size_t i) const
{
    // return *(vect_[(i<0 || i>=size_) ? 0 : i]);
    return *(vect_[i]);
}

void GLFVertices::resize(size_t newsize)
{
size_t cap = (newsize * 9 / 64 + 1) * 8;

    if ((newsize || vect_) && (newsize!=size_)) {
        if (cap != capacity_) {
            for (unsigned int i=newsize; i<size_; i++) {
                delete vect_[i];
            }
            capacity_ = cap;
            //
            // if vect_ == NULL --> realloc == malloc
            //
            vect_ = (GLFVertex **)realloc (
                vect_, capacity_ * sizeof(GLFVertex *)
            );
        }
    }
    size_ = newsize;
}

void GLFVertices::push_back(const GLFVertex& p)
{
    if (size_ == capacity_) {
        resize(size_ + 1);
    } else {
        size_++;
    }
    vect_[size_-1] = new GLFVertex(p);
}

void GLFVertices::clear()
{
    resize(0);
}

GLFVertex& GLFVertices::back()
{
    return *(vect_[size_-1]);
}
