/*
 * glFeedback, an OpenGL Feedback processing Library
 * Copyright (C) 2003 Francesco Bradascio
 *
 * E-mail: fbradasc@katamail.com
 * URL: http://digilander.libero.it/fbradasc/GL_FEEDBACK.html
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * This work is mostly based on the GL2PS Library which is:
 *
 * Copyright (C) 1999-2003 Christophe Geuzaine
 * E-mail: geuz@geuz.org
 * URL: http://www.geuz.org/gl2ps/
 *
 * GL2PS Contributor(s):
 *   Michael Sweet <mike@easysw.com>
 *   Marc Ume <marc.ume@digitalgraphics.be>
 *   Jean-Francois Remacle <remacle@scorec.rpi.edu>
 *   Bart Kaptein <B.L.Kaptein@lumc.nl>
 *   Quy Nguyen-Dai<quy@vnilux.com>
 *   Sam Buss <sbuss@ucsd.edu>
 *   Shane Hill <Shane.Hill@dsto.defence.gov.au>
 *   Romain Boman <r_boman@yahoo.fr>
 *   Rouben Rostamian <rostamian@umbc.edu>
 *   Diego Santa Cruz <Diego.SantaCruz@epfl.ch>
 *   Shahzad Muzaffar <Shahzad.Muzaffar@cern.ch>
 *   Lassi Tuura <lassi.tuura@cern.ch>
 *   Guy Barrand <barrand@lal.in2p3.fr>
 */

#include <GLF/GLFplane.h>

GLFplane::GLFplane()
{
    reset();
}

GLFplane::GLFplane(GLfloat *plane)
{
    set(plane);
}

GLFplane::GLFplane(const GLFplane& plane)
{
    set((GLfloat*)plane);
}

GLFplane::GLFplane(GLfloat a, GLfloat b, GLfloat c, GLfloat d)
{
    set(a,b,c,d);
}

void GLFplane::set(GLfloat a, GLfloat b, GLfloat c, GLfloat d)
{
    vect_[0] = a;
    vect_[1] = b;
    vect_[2] = c;
    vect_[3] = d;
}

void GLFplane::set(const GLfloat *plane)
{
    for (int i=0; i<4; i++) {
        vect_[i] = plane[i];
    }
}

void GLFplane::reset()
{
    set(0.0,0.0,0.0,0.0);
}

GLfloat GLFplane::compare(const GLFxyz& pt) const
{
    return (
        vect_[0]*pt[0] +
        vect_[1]*pt[1] +
        vect_[2]*pt[2] +
        vect_[3]
    );
}

GLint GLFplane::fromPoints(GLFxyz &a, GLFxyz &b)
{
GLfloat n;

    vect_[0] = b[1] - a[1];
    vect_[1] = a[0] - b[0];
    n = sqrt(vect_[0]*vect_[0] + vect_[1]*vect_[1]);
    vect_[2] = 0.0;
    if (n != 0.0) {
        vect_[0] /= n;
        vect_[1] /= n;
        vect_[3] = -vect_[0]*a[0]-vect_[1]*a[1];
        return 1;
    } else {
        vect_[0] = -1.0;
        vect_[1] = 0.0;
        vect_[3] = a[0];
        return 0;
    }
}

GLFplane::operator GLfloat* () const
{
    return (GLfloat* const)vect_;
}

GLFplane::operator GLfloat* ()
{
    return vect_;
}

GLFplane& GLFplane::operator = (const GLfloat* v)
{
    set(v);
    return *this;
}

GLFplane& GLFplane::operator = (const GLFplane& v)
{
    set((GLfloat*)v);
    return *this;
}

bool GLFplane::operator != (const GLfloat* v) const
{
    for (int i=0; i<4; i++) {
        if (vect_[i] != v[i]) {
            return true;
        }
    }
    return false;
}

bool GLFplane::operator != (const GLFplane& v) const
{
    for (int i=0; i<4; i++) {
        if (vect_[i] != v[i]) {
            return true;
        }
    }
    return false;
}
