/*
 * glFeedback, an OpenGL Feedback processing Library
 * Copyright (C) 2003 Francesco Bradascio
 *
 * E-mail: fbradasc@katamail.com
 * URL: http://digilander.libero.it/fbradasc/GL_FEEDBACK.html
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * This work is mostly based on the GL2PS Library which is:
 *
 * Copyright (C) 1999-2003 Christophe Geuzaine
 * E-mail: geuz@geuz.org
 * URL: http://www.geuz.org/gl2ps/
 *
 * GL2PS Contributor(s):
 *   Michael Sweet <mike@easysw.com>
 *   Marc Ume <marc.ume@digitalgraphics.be>
 *   Jean-Francois Remacle <remacle@scorec.rpi.edu>
 *   Bart Kaptein <B.L.Kaptein@lumc.nl>
 *   Quy Nguyen-Dai<quy@vnilux.com>
 *   Sam Buss <sbuss@ucsd.edu>
 *   Shane Hill <Shane.Hill@dsto.defence.gov.au>
 *   Romain Boman <r_boman@yahoo.fr>
 *   Rouben Rostamian <rostamian@umbc.edu>
 *   Diego Santa Cruz <Diego.SantaCruz@epfl.ch>
 *   Shahzad Muzaffar <Shahzad.Muzaffar@cern.ch>
 *   Lassi Tuura <lassi.tuura@cern.ch>
 *   Guy Barrand <barrand@lal.in2p3.fr>
 */

#include <GLF/GLFrgba.h>

GLFrgba::GLFrgba()
{
    reset();
}

GLFrgba::GLFrgba(GLfloat *rgba)
{
    set(rgba);
}

GLFrgba::GLFrgba(const GLFrgba& rgba)
{
    set((GLfloat*)rgba);
}

GLFrgba::GLFrgba(GLfloat r, GLfloat g, GLfloat b, GLfloat a)
{
    set(r,g,b,a);
}

void GLFrgba::set(GLfloat r, GLfloat g, GLfloat b, GLfloat a)
{
    vect_[0] = r;
    vect_[1] = g;
    vect_[2] = b;
    vect_[3] = a;
}

void GLFrgba::set(const GLfloat *rgba)
{
    for (int i=0; i<4; i++) {
        vect_[i] = rgba[i];
    }
}

void GLFrgba::reset()
{
    set(-1.0,-1.0,-1.0,-1.0);
}

GLFrgba::operator GLfloat* () const
{
    return (GLfloat* const)vect_;
}

GLFrgba::operator GLfloat* ()
{
    return vect_;
}

GLFrgba& GLFrgba::operator = (const GLfloat* v)
{
    set(v);
    return *this;
}

GLFrgba& GLFrgba::operator = (const GLFrgba& v)
{
    set((GLfloat*)v);
    return *this;
}

bool GLFrgba::operator != (const GLfloat* v) const
{
    for (int i=0; i<4; i++) {
        if (vect_[i] != v[i]) {
            return true;
        }
    }
    return false;
}

bool GLFrgba::operator != (const GLFrgba& v) const
{
    for (int i=0; i<4; i++) {
        if (vect_[i] != v[i]) {
            return true;
        }
    }
    return false;
}
