/*
 * glFeedback, an OpenGL Feedback processing Library
 * Copyright (C) 2003 Francesco Bradascio
 *
 * E-mail: fbradasc@katamail.com
 * URL: http://digilander.libero.it/fbradasc/GL_FEEDBACK.html
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * This work is mostly based on the GL2PS Library which is:
 *
 * Copyright (C) 1999-2003 Christophe Geuzaine
 * E-mail: geuz@geuz.org
 * URL: http://www.geuz.org/gl2ps/
 *
 * GL2PS Contributor(s):
 *   Michael Sweet <mike@easysw.com>
 *   Marc Ume <marc.ume@digitalgraphics.be>
 *   Jean-Francois Remacle <remacle@scorec.rpi.edu>
 *   Bart Kaptein <B.L.Kaptein@lumc.nl>
 *   Quy Nguyen-Dai<quy@vnilux.com>
 *   Sam Buss <sbuss@ucsd.edu>
 *   Shane Hill <Shane.Hill@dsto.defence.gov.au>
 *   Romain Boman <r_boman@yahoo.fr>
 *   Rouben Rostamian <rostamian@umbc.edu>
 *   Diego Santa Cruz <Diego.SantaCruz@epfl.ch>
 *   Shahzad Muzaffar <Shahzad.Muzaffar@cern.ch>
 *   Lassi Tuura <lassi.tuura@cern.ch>
 *   Guy Barrand <barrand@lal.in2p3.fr>
 */

#include <GLF/GLFxyz.h>

GLFxyz::GLFxyz()
{
    reset();
}

GLFxyz::GLFxyz(GLfloat *xyz)
{
    set(xyz);
}

GLFxyz::GLFxyz(const GLFxyz& xyz)
{
    set((GLfloat*)xyz);
}

GLFxyz::GLFxyz(GLfloat x, GLfloat y, GLfloat z)
{
    set(x,y,z);
}

void GLFxyz::set(GLfloat x, GLfloat y, GLfloat z)
{
    vect_[0] = x;
    vect_[1] = y;
    vect_[2] = z;
}

void GLFxyz::set(const GLfloat *xyz)
{
    for (int i=0; i<3; i++) {
        vect_[i] = xyz[i];
    }
}

void GLFxyz::reset()
{
    set(0.0,0.0,0.0);
}

GLFxyz::operator GLfloat* () const
{
    return (GLfloat* const)vect_;
}

GLFxyz::operator GLfloat* ()
{
    return vect_;
}

GLFxyz& GLFxyz::operator = (const GLfloat* v)
{
    set(v);
    return *this;
}

GLFxyz& GLFxyz::operator = (const GLFxyz& v)
{
    set((GLfloat*)v);
    return *this;
}

bool GLFxyz::operator != (const GLfloat* v) const
{
    for (int i=0; i<3; i++) {
        if (vect_[i] != v[i]) {
            return true;
        }
    }
    return false;
}

bool GLFxyz::operator != (const GLFxyz& v) const
{
    for (int i=0; i<3; i++) {
        if (vect_[i] != v[i]) {
            return true;
        }
    }
    return false;
}
