/*
 * glFeedback, an OpenGL Feedback processing Library
 * Copyright (C) 2003 Francesco Bradascio
 *
 * E-mail: fbradasc@katamail.com
 * URL: http://digilander.libero.it/fbradasc/GL_FEEDBACK.html
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * This work is mostly based on the GL2PS Library which is:
 *
 * Copyright (C) 1999-2003 Christophe Geuzaine 
 * E-mail: geuz@geuz.org
 * URL: http://www.geuz.org/gl2ps/
 *
 * GL2PS Contributor(s):
 *   Michael Sweet <mike@easysw.com>
 *   Marc Ume <marc.ume@digitalgraphics.be>
 *   Jean-Francois Remacle <remacle@scorec.rpi.edu>
 *   Bart Kaptein <B.L.Kaptein@lumc.nl>
 *   Quy Nguyen-Dai<quy@vnilux.com>
 *   Sam Buss <sbuss@ucsd.edu>
 *   Shane Hill <Shane.Hill@dsto.defence.gov.au>
 *   Romain Boman <r_boman@yahoo.fr>
 *   Rouben Rostamian <rostamian@umbc.edu>
 *   Diego Santa Cruz <Diego.SantaCruz@epfl.ch>
 *   Shahzad Muzaffar <Shahzad.Muzaffar@cern.ch>
 *   Lassi Tuura <lassi.tuura@cern.ch>
 *   Guy Barrand <barrand@lal.in2p3.fr>
 */


 */

#include <GLF/glFeedback.h>
#include <GLF/GLFContext.h>
#include <GLF/GLFBuffers.h>

GLFContext *context=0;

void glfSetContext(GLFContext *ctx, GLsizei buffsize)
{
    context = ctx;
    if (context && (!buffsize || context->setFeedbackBuffer(buffsize))) {
        glFeedbackBuffer (
            context->buffsize,
            GL_3D_COLOR,
            context->feedback->buffer
        );
        glRenderMode(GL_FEEDBACK);
    }
}

void glfBeginFrame(GLint x, GLint y, GLsizei width, GLsizei height)
{
    if (context && context->active(true)) {
        GLint vp[4];
        vp[0] = x; vp[1] = y; vp[2] = width; vp[3] = height;
        glRenderMode(GL_FEEDBACK);
        context->beginViewport(vp);
    }
}

void glfEndFrame()
{
    if (context && context->active()) {
        //
        // close all the viewports open at lower levels
        //
        context->endViewport(true);
        glRenderMode(GL_RENDER);
    }
}

void glfViewport(GLint x, GLint y, GLsizei width, GLsizei height)
{
    glViewport(x,y,width,height);
    if (context && context->active()) {
        GLint vp[4];
        vp[0] = x; vp[1] = y; vp[2] = width; vp[3] = height;
        context->beginViewport(vp);
    }
}

void glfDrawPixels (
    GLsizei width , GLsizei height,
    GLenum  format, GLenum  type  ,
    const void *pixels
) {
    glDrawPixels(width, height, format, type, pixels);
    if (context && context->active()) {
        context->drawPixels(width, height, format, type, pixels);
    }
}

void glfPointSize(GLfloat size)
{
    glPointSize(size);
    if (context && context->active()) {
        context->pointSize(size);
    }
}

void glfLineWidth(GLfloat width)
{
    glLineWidth(width);
    if (context && context->active()) {
        context->lineWidth(width);
    }
}

void glfLineStipple(GLint factor, GLushort pattern)
{
    glLineStipple(factor, pattern);
    if (context && context->active()) {
        context->lineStipple(factor, pattern);
    }
}

void glfPolygonStipple(const GLubyte *mask)
{
    glPolygonStipple(mask);
    if (context && context->active()) {
        context->polygonStipple(mask);
    }
}

void glfEnable(GLenum cap)
{
    glEnable(cap);
    if (context && context->active()) {
        context->enable(cap);
    }
}

void glfDisable(GLenum cap)
{
    glDisable(cap);
    if (context && context->active()) {
        context->disable(cap);
    }
}
