/*
 * glFeedback, an OpenGL Feedback processing Library
 * Copyright (C) 2003 Francesco Bradascio
 *
 * E-mail: fbradasc@katamail.com
 * URL: http://digilander.libero.it/fbradasc/GL_FEEDBACK.html
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * This example is a glFeedback adaptation of the GL2PS's gl2psTest.c example
 *
 * This work is mostly based on the GL2PS Library which is:
 *
 * Copyright (C) 1999-2003 Christophe Geuzaine
 * E-mail: geuz@geuz.org
 * URL: http://www.geuz.org/gl2ps/
 *
 * GL2PS Contributor(s):
 *   Michael Sweet <mike@easysw.com>
 *   Marc Ume <marc.ume@digitalgraphics.be>
 *   Jean-Francois Remacle <remacle@scorec.rpi.edu>
 *   Bart Kaptein <B.L.Kaptein@lumc.nl>
 *   Quy Nguyen-Dai<quy@vnilux.com>
 *   Sam Buss <sbuss@ucsd.edu>
 *   Shane Hill <Shane.Hill@dsto.defence.gov.au>
 *   Romain Boman <r_boman@yahoo.fr>
 *   Rouben Rostamian <rostamian@umbc.edu>
 *   Diego Santa Cruz <Diego.SantaCruz@epfl.ch>
 *   Shahzad Muzaffar <Shahzad.Muzaffar@cern.ch>
 *   Lassi Tuura <lassi.tuura@cern.ch>
 *   Guy Barrand <barrand@lal.in2p3.fr>
 */

#include <GLF/gl.h> // Include this instead of GL/gl.h to redefine some gl
                    // routines for the GL Feedback rendering
#include <GL/glut.h>
#include <string.h>
#include <stdlib.h>
#include <GLF/GLFPostscript.h> // the Postscript renderer

#include <sys/time.h>

static float rotation = -60.;
static GLsizei window_w = 0; 
static GLsizei window_h = 0;
static GLboolean display_multi = 1;
static GLboolean teapot = 0;
static char *pixmap[] = {
    "................................................................",
    "...................++++++++++++++++++++++++++...................",
    "..............+++++++++++++++++++++++++++++++++++++.............",
    "........++++++++++++@@@@@@@@@@@@@@@@@@@@@@@@++++++++++++........",
    "........++++++@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@++++++........",
    "........++@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@++........",
    "........++@@@@++++@@@+@@@@@@@++++++@@++++++@@@+@@@@+@@++........",
    "........++@@@+@@@@+@@+@@@@@@@+@@@@@@@+@@@@@+@@+@@@+@@@++........",
    "........++@@+@@@@@@@@+@@@@@@@+@@@@@@@+@@@@@+@@+@@+@@@@++........",
    "........++@@+@@@+++@@+@@@@@@@++++@@@@++++++@@@+++@@@@@++........",
    "........++@@+@@@@@+@@+@@@@@@@+@@@@@@@+@@@@@+@@+@@+@@@@++........",
    "........++@@@+@@@@+@@+@@@@@@@+@@@@@@@+@@@@@+@@+@@@+@@@++........",
    "........++@@@@+++++@@++++++@@+@@@@@@@++++++@@@+@@@@+@@++........",
    "........++@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@++........",
    "........++@@@@@@@@@++++++++++++++++++++++++++@@@@@@@@@++........",
    "........++@@@+++++++@@@@@@@@@@@+++++++++++++++++++@@@@++........",
    "........++++++@@+@@@+@@@++@@@@@+++++++++++++++++++++++++........",
    "........++@@@@@+@@@+@@@+@@@@+@@+++++++++++++++++++++++++........",
    "........++@@@@+@@@+@@+++++++@@@+++++++++++++++++++++++++........",
    "........++@@@@+++++++@@@@@@++@@++#####################++........",
    "........++@+++++@@@@@@@@@@@@@@@++#####################++........",
    "........++@+++@@@@@@@@@@@@@@@@@++#####################++........",
    "........++@++@@@@@@@+@@+@@@@@@@++#####################++........",
    "........++@@@@@+@@@+@@+@@@@@@@@++#####################++........",
    "........++@@@@+@@@+@@+@@++++++++++++++++++++++++++++++++........",
    "........++@@@@+@+++++++++@++++@+@+@+++++++++++++++++++++........",
    "........++@@@@+++@@@@@@@+@++@+++@+@+@+@+++++++++++++++++........",
    "........++@++++@@@@@@@@@@+@+++@+@+@+++++++++++++++++++++........",
    "........++@+++@@@@@@@@@@+@@@@@@@@@@@@@@+++++++++++++++++........",
    "........++@++@@@@@@+@@+@+@@@+++@@@@@@@@+++++++++++++++++........",
    "........++@@@@@+@@+@@+@@+@@+++++@@@@@@@+##############++........",
    "........++@@@@+@@+@@+@@++@@+@+++@@@@@+@+##############++........",
    "........++@@@@+@+++++++@+@@@@+++@@@@++@+##############++........",
    "........++@++++++@@@@@@++@@@@+++@@@@++@+##############++........",
    "........++@+++@@@@@@@@@@+@@@+++++@@@+@@+##############++........",
    "........++@++@@@@@@@@@@@+@+++++++++++@@+##############++........",
    "........+++++++++++++++++@+@+++++++++@@+++++++++++++++++........",
    "........+++++++++++++++++@+@+@@@@++++@@+@@@@@@@@@@@@@@++........",
    "........+++++++++++++++++@@@+@@@@@+@+@@+@@@+@@+@@@@@@@++........",
    ".........++++++++++++++++@@@@+@@@++@++@+@@+@@+@@@+@@@++.........",
    ".........++++++++++++++++@@@@@@@@+@@+@@+@+@@+@@@+@@@@++.........",
    ".........+++++++++++++++++@@@@@@+@@+@@+@++++++++++@@@++.........",
    "..........++##############+@@@@@@@@@@++++@@@@@@@@++@++..........",
    "..........++###############+@@@@@@@@++++@@@@@@@@@@@@++..........",
    "...........++###############++++++++@@@@@@@+@@+@@@@++...........",
    "...........++##################++@@@@@+@@@+@@+@@+@@++...........",
    "............++#################++@@@@+@@@+@@+@@+@@++............",
    "............++#################++@@@+@@++++++++@@@++............",
    ".............++++++++++++++++++++@@@++++@@@@@@++@++.............",
    "..............+++++++++++++++++++@@+++@@@@@@@@@@@+..............",
    "..............+++++++++++++++++++@+++@@@@@@@@@@@++..............",
    "...............++++++++++++++++++@++@@@@+@@@@@@++...............",
    "................+++++++++++++++++@@@+@@+@@+@@@++................",
    ".................++++++++++++++++@@+@@+@@+@@@++.................",
    "..................++###########++@@+++++++@@++..................",
    "...................++##########++@+++@@@@+@++...................",
    "....................++#########++@++@@@@@@++....................",
    ".....................+++#######++@++@@@@+++.....................",
    "......................++++#####++@@@@@++++......................",
    "........................++++###++@@@++++........................",
    "..........................++++#++@++++..........................",
    "............................++++++++............................",
    "..............................++++..............................",
    "................................................................"
};

void init(void)
{
float pos[3] = {0.,0.,1000.};

    glEnable(GL_DEPTH_TEST);
    glShadeModel(GL_SMOOTH);
    glLightfv(GL_LIGHT0, GL_POSITION, pos);
    glEnable(GL_LIGHT0);
    glEnable(GL_SCISSOR_TEST);
}

void triangles(void)
{
    /* two intersecting triangles */
    glDisable(GL_LIGHTING);
    glBegin(GL_TRIANGLES);
    
        glColor3f(1., 0., 0.);
        glVertex3f(-1., 1., 0.);
        glColor3f(1., 1., 0.);
        glVertex3f(-1., 0., 0.);
        glColor3f(1., 0., 1.);
        glVertex3f(1., 0., 0.2);
    
        glColor3f(0., 1., 0.);
        glVertex3f(1., 0., 0.);
        glColor3f(0., 1., 1.);
        glVertex3f(1., 0.5, 0.);
        glColor3f(0., 1., 1.);
        glVertex3f(-1., 0.5, 0.1);

    glEnd();
}

void objects(void)
{
    glEnable(GL_LIGHTING);
    glPushMatrix();
    glRotatef(rotation, 2., 0., 1.);
    if (teapot) {
      glutSolidTeapot(0.7);
    } else {
      glutSolidTorus(0.3, 0.6, 20, 20);
    }
    glPopMatrix();
}

void printstring(char *string)
{
int len, i;

    len = (int)strlen(string);
    for (i = 0; i < len; i++) {
        glutBitmapCharacter(GLUT_BITMAP_HELVETICA_12, string[i]);
    }
}

void text()
{
double x = -1.25, y = -0.58, dy = 0.13;

    glDisable(GL_LIGHTING);
    glColor3f(1., 1., 0.);

    glRasterPos2d(x, y); y -= dy;
    printstring("Press:");
    glRasterPos2d(x, y); y -= dy;
    printstring("  s: to save the images");
    glRasterPos2d(x, y); y -= dy;
    printstring("  t: to alternate between teapot and torus");
    glRasterPos2d(x, y); y -= dy;
    printstring("  v: to alternate between single and multiple viewport modes");
    glRasterPos2d(x, y); y -= dy;
    printstring("  q: to quit");
    glRasterPos2d(x, y); y -= dy;
    printstring("Click and move the mouse to rotate the objects");
}

void cube(void)
{
    glDisable(GL_LIGHTING);
    glLineWidth(4.0);
    glColor3d (1.0,0.0,0.);   
    glBegin(GL_POLYGON);
        glVertex3d( 0.5,-0.5,0.5);
        glVertex3d( 0.5, 0.5,0.5);
        glVertex3d(-0.5, 0.5,0.5);
        glVertex3d(-0.5,-0.5,0.5);
    glEnd();

    glColor3d (0.0,1.0,0.);   
    glBegin(GL_POLYGON);
        glVertex3d( 0.5,-0.5,-0.5);
        glVertex3d( 0.5, 0.5,-0.5);
        glVertex3d(-0.5, 0.5,-0.5);
        glVertex3d(-0.5,-0.5,-0.5);
    glEnd();

    glColor3d (1.0,1.0,0.);   
    glEnable(GL_LINE_STIPPLE);
    glLineWidth(2);
    glLineStipple(1,0x0c0f);
    glBegin(GL_LINES);
        glVertex3d( 0.5,-0.5, 0.5);
        glVertex3d( 0.5,-0.5,-0.5);
        glVertex3d( 0.5, 0.5, 0.5);
        glVertex3d( 0.5, 0.5,-0.5);
        glVertex3d(-0.5, 0.5, 0.5);
        glVertex3d(-0.5, 0.5,-0.5);
        glVertex3d(-0.5,-0.5, 0.5);
        glVertex3d(-0.5,-0.5,-0.5);
    glEnd();
}

void image()
{
int w = 64, h = 64, row, col, pos = 0;
float *pixels, r = 0., g = 0., b = 0.;

    /* Fill a pixmap (each pixel contains three floats defining an RGB color) */
    pixels = (float*)malloc(3*w*h*sizeof(float));

    for (row = w-1; row >= 0; row--) {
        for (col = 0; col < h; col++) {
            switch (pixmap[row][col]) {
                case '.' : r = 255.; g = 255.; b = 255.; break;
                case '+' : r = 0.  ; g = 0.  ; b = 0.  ; break;
                case '@' : r = 255.; g = 209.; b = 0.  ; break;
                case '#' : r = 255.; g = 0.  ; b = 0.  ; break;
            }
            r /= 255.; g /= 255.; b /= 255.; 
            pixels[pos] = r; pos++;
            pixels[pos] = g; pos++;
            pixels[pos] = b; pos++;
        }
    }
    glRasterPos2f(-0.8, -0.8);
    glDrawPixels((GLsizei)w, (GLsizei)h, GL_RGB, GL_FLOAT, pixels);

    free(pixels);
}

/* A simple drawing function, using the default viewport */
void draw_single(void)
{
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    triangles();
    objects();
    text();
    glFlush();
}

/* A more complex drawing function, using 2 separate viewports */
void draw_multi(void)
{
GLint viewport[4];

    glScissor(0, 0, window_w, window_h);
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    /* First viewport with triangles, teapot or torus and text */
    glViewport((GLsizei) (window_w * 0.05), (GLsizei) (window_h * 0.525) , 
  	     (GLsizei) (window_w * 0.9), (GLsizei) (window_h * 0.45));
    glScissor((GLsizei) (window_w * 0.05), (GLsizei) (window_h * 0.525) , 
  	    (GLsizei) (window_w * 0.9), (GLsizei) (window_h * 0.45));
    glClearColor(0.3, 0.3, 0.3, 0.);

        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
        glMatrixMode(GL_PROJECTION);
        glLoadIdentity();
        glOrtho(-1.3,1.3, -1.3,1.3, -1.3,1.3);
        glMatrixMode(GL_MODELVIEW);
      
        triangles();
        objects();
        text();

    /* Second viewport with cube and image */
    glViewport((GLsizei) (window_w * 0.05), (GLsizei) (window_h * 0.025) , 
  	     (GLsizei) (window_w * 0.9), (GLsizei) (window_h * 0.45));
    glScissor((GLsizei) (window_w * 0.05), (GLsizei) (window_h * 0.025) , 
  	     (GLsizei) (window_w * 0.9), (GLsizei) (window_h * 0.45));
    glClearColor(0.5, 0.5, 0.5, 0.);

        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
        glMatrixMode(GL_PROJECTION);
        glLoadIdentity();
        glOrtho(-1.3,1.3, -1.3,1.3, -1.3,1.3);
        glMatrixMode(GL_MODELVIEW);
    
        glPushMatrix();
            glRotatef(rotation, 1., 1., 1.);
            cube();
            image();
        glPopMatrix();

    glClearColor(0.7, 0.7, 0.7, 0.);
    glFlush();
}

void display(void)
{
    if(display_multi){
        draw_multi();
    } else{
        draw_single();
    }
}

void reshape(int w, int h)
{
    window_w = w;
    window_h = h;

    glViewport(0, 0, (GLsizei) window_w, (GLsizei) window_h);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glOrtho(-1.3,1.3, -1.3,1.3, -1.3,1.3);
    glMatrixMode(GL_MODELVIEW);

    glClearColor(0.7, 0.7, 0.7, 0.);
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
}

void writeps()
{
GLint viewport[4];
GLint buffsize;

    viewport[0] = 0;
    viewport[1] = 0;
    viewport[2] = window_w;
    viewport[3] = window_h;
    //
    // Create the rendering context
    //
    GLFPostscript *context = new GLFPostscript (
        /* Output file: */ "out.ps",
        /* Screen size: */ viewport,
        /* Title      : */ "test",
        /* Autor      : */ "fbradasc",
        /* Producer   : */ "glfTest",
        /* Copyright  : */ "fbradasc (c) 2003"
    );
    //
    // Setup the page format
    //
    context->pageSetup (
        /* Orientation              : */ GLFPostscript::PORTRAIT,
        /* Page width  [mm]         : */ 210.0,
        /* Page height [mm]         : */ 297.0,
        /* Page border [mm]         : */  10.0,
        /* [mm] -> [points] @ 72 dpi: */ 72.0 / 25.4
    );

    buffsize = 0;
    do {
        buffsize += 1024*1024;
        //
        // Activate the rendering context
        //
        glfSetContext(context,buffsize);
        //
        // printing 1st page
        //
            //
            // Setup the rendering options
            //
            context->setOptions (
                GLF_OPTION_FLAG(GLF_BEST_ROOT) | 
                GLF_OPTION_FLAG(GLF_DRAW_BACKGROUND)
            );
            //
            // Open the page
            //
            glfBeginFrame(viewport[0],viewport[1],viewport[2],viewport[3]);
                //
                // draw the scene
                //
                display();
            //
            // Close the page
            //
            glfEndFrame();
        //
        // Extract the primitives from the GL Feedback buffer
        //
        if (context->fetchPrimitives() != GLF_OVERFLOW) {
            //
            // render the 2nd page
            //
            context->setOptions (
                GLF_OPTION_FLAG(GLF_OCCLUSION_CULL) |
                GLF_OPTION_FLAG(GLF_BEST_ROOT)      |
                GLF_OPTION_FLAG(GLF_DRAW_BACKGROUND)
            );
            glfBeginFrame(viewport[0],viewport[1],viewport[2],viewport[3]);
                display();
            glfEndFrame();
            //
            // Restore the normal rendering mode
            //
            glfSetContext(0,0);
            //
            // Redisplay the scene
            //
            display();
        }
    } while (context->fetchPrimitives() == GLF_OVERFLOW);

    printf("\nglFeedback saving all pages on out.ps...\n");
    //
    // Saving the primitives into the PS file
    //
    context->printPrimitives();
    delete context;

    printf("glFeedback done with all images in out.ps\n");
}

void keyboard(unsigned char key, int x, int y)
{
    switch (key) {
    case 27:
    case 'q':
            exit(0);
        break;
    case 't':
            teapot = !teapot;
            display();
        break;
    case 'v':
            display_multi = !display_multi;
            if (display_multi) {
                glEnable(GL_SCISSOR_TEST);
            } else{
                glDisable(GL_SCISSOR_TEST);
            }
            reshape(window_w, window_h);
            display();
        break;
    case 's':
            writeps();
        break;
    }
}

void motion(int x, int y)
{
    rotation += 10.;
    display();
}

int main(int argc, char **argv)
{
    glutInit(&argc, argv);
    glutInitDisplayMode(GLUT_SINGLE | GLUT_DEPTH);
    glutInitWindowSize(400, 600);
    glutInitWindowPosition(100, 100);
    glutCreateWindow(argv[0]);
    init();
    glutDisplayFunc(display);
    glutReshapeFunc(reshape);
    glutKeyboardFunc(keyboard);
    glutMotionFunc(motion);
    glutMainLoop();
    return 0;
}
